@@ -1,9 +1,9 @@
 /*
 ---------------------------------------------------------------------------
-Open Asset Import Library (ASSIMP)
+Open Asset Import Library (assimp)
 ---------------------------------------------------------------------------
 
-Copyright (c) 2006-2010, ASSIMP Development Team
+Copyright (c) 2006-2012, assimp team
 
 All rights reserved.
 
@@ -20,10 +20,10 @@ conditions are met:
   following disclaimer in the documentation and/or other
   materials provided with the distribution.
 
-* Neither the name of the ASSIMP team, nor the names of its
+* Neither the name of the assimp team, nor the names of its
   contributors may be used to endorse or promote products
   derived from this software without specific prior
-  written permission of the ASSIMP Development Team.
+  written permission of the assimp team.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
@@ -48,7 +48,6 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 // internal headers
 #include "LWOLoader.h"
-#include "MaterialSystem.h"
 #include "StringComparison.h"
 #include "SGSpatialSort.h"
 #include "ByteSwap.h"
@@ -57,6 +56,19 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 using namespace Assimp;
 
+static const aiImporterDesc desc = {
+	"LightWave/Modo Object Importer",
+	"",
+	"",
+	"http://www.newtek.com/lightwave.html\nhttp://www.luxology.com/modo/",
+	aiImporterFlags_SupportTextFlavour,
+	0,
+	0,
+	0,
+	0,
+	"lwo lxo"
+};
+
 // ------------------------------------------------------------------------------------------------
 // Constructor to be privately used by Importer
 LWOImporter::LWOImporter()
@@ -72,8 +84,9 @@ LWOImporter::~LWOImporter()
 bool LWOImporter::CanRead( const std::string& pFile, IOSystem* pIOHandler, bool checkSig) const
 {
 	const std::string extension = GetExtension(pFile);
-	if (extension == "lwo" || extension == "lxo")
+	if (extension == "lwo" || extension == "lxo") {
 		return true;
+	}
 
 	// if check for extension is not enough, check for the magic tokens 
 	if (!extension.length() || checkSig) {
@@ -91,10 +104,17 @@ bool LWOImporter::CanRead( const std::string& pFile, IOSystem* pIOHandler, bool
 void LWOImporter::SetupProperties(const Importer* pImp)
 {
 	configSpeedFlag  = ( 0 != pImp->GetPropertyInteger(AI_CONFIG_FAVOUR_SPEED,0) ? true : false);
-	configLayerIndex = pImp->GetPropertyInteger (AI_CONFIG_IMPORT_LWO_ONE_LAYER_ONLY,0xffffffff); 
+	configLayerIndex = pImp->GetPropertyInteger (AI_CONFIG_IMPORT_LWO_ONE_LAYER_ONLY,UINT_MAX); 
 	configLayerName  = pImp->GetPropertyString  (AI_CONFIG_IMPORT_LWO_ONE_LAYER_ONLY,"");
 }
 
+// ------------------------------------------------------------------------------------------------
+// Get list of file extensions
+const aiImporterDesc* LWOImporter::GetInfo () const
+{
+	return &desc;
+}
+
 // ------------------------------------------------------------------------------------------------
 // Imports the given file into the given scene structure. 
 void LWOImporter::InternReadFile( const std::string& pFile, 
@@ -143,6 +163,7 @@ void LWOImporter::InternReadFile( const std::string& pFile,
 	mLayers->push_back(Layer());
 	mCurLayer = &mLayers->back();
 	mCurLayer->mName = "<LWODefault>";
+	mCurLayer->mIndex = -1;
 
 	// old lightwave file format (prior to v6)
 	if (AI_LWO_FOURCC_LWOB == fileType)	{
@@ -181,8 +202,14 @@ void LWOImporter::InternReadFile( const std::string& pFile,
 		// The newer lightwave format allows the user to configure the
 		// loader that just one layer is used. If this is the case
 		// we need to check now whether the requested layer has been found.
-		if (0xffffffff != configLayerIndex && configLayerIndex > mLayers->size())
-			throw DeadlyImportError("LWO2: The requested layer was not found");
+		if (UINT_MAX != configLayerIndex) {
+			unsigned int layerCount = 0;
+			for(std::list<LWO::Layer>::iterator itLayers=mLayers->begin(); itLayers!=mLayers->end(); itLayers++)
+				if (!itLayers->skip)
+					layerCount++;
+			if (layerCount!=2)
+				throw DeadlyImportError("LWO2: The requested layer was not found");
+		}
 
 		if (configLayerName.length() && !hasNamedLayer)	{
 			throw DeadlyImportError("LWO2: Unable to find the requested layer: " 
@@ -196,11 +223,11 @@ void LWOImporter::InternReadFile( const std::string& pFile,
 
 	// now process all layers and build meshes and nodes
 	std::vector<aiMesh*> apcMeshes;
-	std::vector<aiNode*> apcNodes;
-	apcNodes. reserve(mLayers->size());
+	std::map<uint16_t, aiNode*> apcNodes;
+
 	apcMeshes.reserve(mLayers->size()*std::min(((unsigned int)mSurfaces->size()/2u), 1u));
 
-	unsigned int iDefaultSurface = 0xffffffff; // index of the default surface
+	unsigned int iDefaultSurface = UINT_MAX; // index of the default surface
 	for (LayerList::iterator lit = mLayers->begin(), lend = mLayers->end();lit != lend;++lit)	{
 		LWO::Layer& layer = *lit;
 		if (layer.skip)
@@ -225,10 +252,10 @@ void LWOImporter::InternReadFile( const std::string& pFile,
 				if (idx >= mTags->size())
 				{
 					DefaultLogger::get()->warn("LWO: Invalid face surface index");
-					idx = 0xffffffff;
+					idx = UINT_MAX;
 				}
-				if(0xffffffff == idx || 0xffffffff == (idx = _mMapping[idx]))	{
-					if (0xffffffff == iDefaultSurface)	{
+				if(UINT_MAX == idx || UINT_MAX == (idx = _mMapping[idx]))	{
+					if (UINT_MAX == iDefaultSurface)	{
 						iDefaultSurface = (unsigned int)mSurfaces->size();
 						mSurfaces->push_back(LWO::Surface());
 						LWO::Surface& surf = mSurfaces->back();
@@ -239,7 +266,7 @@ void LWOImporter::InternReadFile( const std::string& pFile,
 				}
 				pSorted[idx].push_back(i);
 			}
-			if (0xffffffff == iDefaultSurface) {
+			if (UINT_MAX == iDefaultSurface) {
 				pSorted.erase(pSorted.end()-1);
 			}
 			for (unsigned int p = 0,i = 0;i < mSurfaces->size();++i)	{
@@ -268,10 +295,12 @@ void LWOImporter::InternReadFile( const std::string& pFile,
 				unsigned int vVColorIndices[AI_MAX_NUMBER_OF_COLOR_SETS];
 
 #if _DEBUG
-				for (unsigned int mui = 0; mui < AI_MAX_NUMBER_OF_TEXTURECOORDS;++mui )
-					vUVChannelIndices[mui] = 0xffffffff;
-				for (unsigned int mui = 0; mui < AI_MAX_NUMBER_OF_COLOR_SETS;++mui )
-					vVColorIndices[mui] = 0xffffffff;
+				for (unsigned int mui = 0; mui < AI_MAX_NUMBER_OF_TEXTURECOORDS;++mui ) {
+					vUVChannelIndices[mui] = UINT_MAX;
+				}
+				for (unsigned int mui = 0; mui < AI_MAX_NUMBER_OF_COLOR_SETS;++mui ) {
+					vVColorIndices[mui] = UINT_MAX;
+				}
 #endif
 
 				FindUVChannels(_mSurfaces[i],sorted,layer,vUVChannelIndices);
@@ -280,8 +309,9 @@ void LWOImporter::InternReadFile( const std::string& pFile,
 				// allocate storage for UV and CV channels
 				aiVector3D* pvUV[AI_MAX_NUMBER_OF_TEXTURECOORDS];
 				for (unsigned int mui = 0; mui < AI_MAX_NUMBER_OF_TEXTURECOORDS;++mui )	{
-					if (0xffffffff == vUVChannelIndices[mui])
+					if (UINT_MAX == vUVChannelIndices[mui]) {
 						break;
+					}
 					
 					pvUV[mui] = mesh->mTextureCoords[mui] = new aiVector3D[mesh->mNumVertices];
 
@@ -294,7 +324,9 @@ void LWOImporter::InternReadFile( const std::string& pFile,
 		
 				aiColor4D* pvVC[AI_MAX_NUMBER_OF_COLOR_SETS];
 				for (unsigned int mui = 0; mui < AI_MAX_NUMBER_OF_COLOR_SETS;++mui)	{
-					if (0xffffffff == vVColorIndices[mui])break;
+					if (UINT_MAX == vVColorIndices[mui]) {
+						break;
+					}
 					pvVC[mui] = mesh->mColors[mui] = new aiColor4D[mesh->mNumVertices];
 				}
 
@@ -317,8 +349,9 @@ void LWOImporter::InternReadFile( const std::string& pFile,
 
 						// process UV coordinates
 						for (unsigned int w = 0; w < AI_MAX_NUMBER_OF_TEXTURECOORDS;++w)	{
-							if (0xffffffff == vUVChannelIndices[w])
+							if (UINT_MAX == vUVChannelIndices[w]) {
 								break;
+							}
 							aiVector3D*& pp = pvUV[w];
 							const aiVector2D& src = ((aiVector2D*)&layer.mUVChannels[vUVChannelIndices[w]].rawData[0])[idx];
 							pp->x = src.x;
@@ -335,8 +368,9 @@ void LWOImporter::InternReadFile( const std::string& pFile,
 
 						// process vertex colors
 						for (unsigned int w = 0; w < AI_MAX_NUMBER_OF_COLOR_SETS;++w)	{
-							if (0xffffffff == vVColorIndices[w])
+							if (UINT_MAX == vVColorIndices[w]) {
 								break;
+							}
 							*pvVC[w] = ((aiColor4D*)&layer.mVColorChannels[vVColorIndices[w]].rawData[0])[idx];
 
 							// If a RGB color map is explicitly requested delete the
@@ -378,7 +412,7 @@ void LWOImporter::InternReadFile( const std::string& pFile,
 		unsigned int num = apcMeshes.size() - meshStart;
 		if (layer.mName != "<LWODefault>" || num > 0) {
 			aiNode* pcNode = new aiNode();
-			apcNodes.push_back(pcNode);
+			apcNodes[layer.mIndex] = pcNode;
 			pcNode->mName.Set(layer.mName);
 			pcNode->mParent = (aiNode*)&layer;
 			pcNode->mNumMeshes = num;
@@ -397,7 +431,7 @@ void LWOImporter::InternReadFile( const std::string& pFile,
 	// The RemoveRedundantMaterials step will clean this up later
 	pScene->mMaterials = new aiMaterial*[pScene->mNumMaterials = (unsigned int)mSurfaces->size()];
 	for (unsigned int mat = 0; mat < pScene->mNumMaterials;++mat)	{
-		MaterialHelper* pcMat = new MaterialHelper();
+		aiMaterial* pcMat = new aiMaterial();
 		pScene->mMaterials[mat] = pcMat;
 		ConvertMaterial((*mSurfaces)[mat],pcMat);
 	}
@@ -517,78 +551,69 @@ void LWOImporter::ComputeNormals(aiMesh* mesh, const std::vector<unsigned int>&
 }
 
 // ------------------------------------------------------------------------------------------------
-void LWOImporter::AddChildren(aiNode* node, uint16_t parent, std::vector<aiNode*>& apcNodes)
-{
-	for (std::vector<aiNode*>::iterator it = apcNodes.begin(); it != apcNodes.end(); ++it)	{
-		if (*it) {
-			LWO::Layer* layer = (LWO::Layer*)(*it)->mParent;
-			if (layer->mParent == parent && layer->mIndex != parent)
-				++node->mNumChildren;
-		}
-	}
-
-	if (node->mNumChildren)	{
-		unsigned int p = 0;
-
-		node->mChildren = new aiNode* [ node->mNumChildren ];
-		for (std::vector<aiNode*>::iterator it = apcNodes.begin(); it != apcNodes.end(); ++it)	{
-			if (*it) {
-				LWO::Layer* layer = (LWO::Layer*)(*it)->mParent;
-				if (layer->mParent == parent && layer->mIndex != parent)	{
-					aiNode* nd = node->mChildren[p++] = *it;
-					nd->mParent = node;
-
-					// fixme: ignore pivot points for the moment
-					//nd->mTransformation.a4 = layer->mPivot.x;
-					//nd->mTransformation.b4 = layer->mPivot.y;
-					//nd->mTransformation.c4 = layer->mPivot.z;
-
-					// recursively add more children
-					(*it) = NULL;
-					AddChildren(nd,layer->mIndex,apcNodes);
-				}
-			}
-		}
-	}
-}
-
-// ------------------------------------------------------------------------------------------------
-void LWOImporter::GenerateNodeGraph(std::vector<aiNode*>& apcNodes)
+void LWOImporter::GenerateNodeGraph(std::map<uint16_t,aiNode*>& apcNodes)
 {
 	// now generate the final nodegraph - generate a root node and attach children
 	aiNode* root = pScene->mRootNode = new aiNode();
 	root->mName.Set("<LWORoot>");
-	AddChildren(root,0,apcNodes);
 
-	// check whether we added all layers with meshes assigned to the output graph.
-	// if not, add them to the root node
-	unsigned int extra = 0;
-	for (std::vector<aiNode*>::iterator it = apcNodes.begin(); it != apcNodes.end(); ++it) {
-		if ((*it) && (*it)->mNumMeshes) 
-			++extra;
+	//Set parent of all children, inserting pivots
+	//std::cout << "Set parent of all children" << std::endl;
+	std::map<uint16_t, aiNode*> mapPivot;
+	for (std::map<uint16_t,aiNode*>::iterator itapcNodes = apcNodes.begin(); itapcNodes != apcNodes.end(); ++itapcNodes) {
+
+		//Get the parent index
+		LWO::Layer* nodeLayer = (LWO::Layer*)(itapcNodes->second->mParent);
+		uint16_t parentIndex = nodeLayer->mParent;
+
+		//Create pivot node, store it into the pivot map, and set the parent as the pivot
+		aiNode* pivotNode = new aiNode();
+		pivotNode->mName.Set("Pivot-"+std::string(itapcNodes->second->mName.data));
+		mapPivot[-(itapcNodes->first+2)] = pivotNode;
+		itapcNodes->second->mParent = pivotNode;
+
+		//Look for the parent node to attach the pivot to
+		if (apcNodes.find(parentIndex) != apcNodes.end()) {
+			pivotNode->mParent = apcNodes[parentIndex];
+		} else {
+			//If not, attach to the root node
+			pivotNode->mParent = root;
+		}
+
+		//Set the node and the pivot node transformation
+		itapcNodes->second->mTransformation.a4 = -nodeLayer->mPivot.x;
+		itapcNodes->second->mTransformation.b4 = -nodeLayer->mPivot.y;
+		itapcNodes->second->mTransformation.c4 = -nodeLayer->mPivot.z;
+		pivotNode->mTransformation.a4 = nodeLayer->mPivot.x;
+		pivotNode->mTransformation.b4 = nodeLayer->mPivot.y;
+		pivotNode->mTransformation.c4 = nodeLayer->mPivot.z;
 	}
 
-	if (extra)	{
-		const unsigned int newSize = extra + pScene->mRootNode->mNumChildren;
-		aiNode** const apcNewNodes = new aiNode*[newSize];
-		if((extra = root->mNumChildren))
-			::memcpy(apcNewNodes,root->mChildren,extra*sizeof(void*));
+	//Merge pivot map into node map
+	//std::cout << "Merge pivot map into node map" << std::endl;
+	for (std::map<uint16_t, aiNode*>::iterator itMapPivot = mapPivot.begin(); itMapPivot != mapPivot.end(); ++itMapPivot) {
+		apcNodes[itMapPivot->first] = itMapPivot->second;
+	}
 
-		aiNode** cc = apcNewNodes+extra;
-		for (std::vector<aiNode*>::iterator it = apcNodes.begin(); it != apcNodes.end(); ++it) {
-			if ((*it) && (*it)->mNumMeshes) {
-				aiNode* nd = *cc++ = *it;
-				nd->mParent = pScene->mRootNode;
-
-				// recursively add more children
-				(*it) = NULL;
-				AddChildren(nd,((LWO::Layer*)nd->mParent)->mIndex,apcNodes);
+	//Set children of all parents
+	apcNodes[-1] = root;
+	for (std::map<uint16_t,aiNode*>::iterator itMapParentNodes = apcNodes.begin(); itMapParentNodes != apcNodes.end(); ++itMapParentNodes) {
+		for (std::map<uint16_t,aiNode*>::iterator itMapChildNodes = apcNodes.begin(); itMapChildNodes != apcNodes.end(); ++itMapChildNodes) {
+			if ((itMapParentNodes->first != itMapChildNodes->first) && (itMapParentNodes->second == itMapChildNodes->second->mParent)) {
+				++(itMapParentNodes->second->mNumChildren);
+			}
+		}
+		if (itMapParentNodes->second->mNumChildren) {
+			itMapParentNodes->second->mChildren = new aiNode* [ itMapParentNodes->second->mNumChildren ];
+			uint16_t p = 0;
+			for (std::map<uint16_t,aiNode*>::iterator itMapChildNodes = apcNodes.begin(); itMapChildNodes != apcNodes.end(); ++itMapChildNodes) {
+				if ((itMapParentNodes->first != itMapChildNodes->first) && (itMapParentNodes->second == itMapChildNodes->second->mParent)) {
+					itMapParentNodes->second->mChildren[p++] = itMapChildNodes->second;
+				}
 			}
 		}
-		delete[] root->mChildren;
-		root->mChildren = apcNewNodes;
-		root->mNumChildren = newSize;
 	}
+
 	if (!pScene->mRootNode->mNumChildren)
 		throw DeadlyImportError("LWO: Unable to build a valid node graph");
 
@@ -612,7 +637,7 @@ void LWOImporter::GenerateNodeGraph(std::vector<aiNode*>& apcNodes)
 void LWOImporter::ResolveTags()
 {
 	// --- this function is used for both LWO2 and LWOB
-	mMapping->resize(mTags->size(),0xffffffff);
+	mMapping->resize(mTags->size(), UINT_MAX);
 	for (unsigned int a = 0; a  < mTags->size();++a)	{
 
 		const std::string& c = (*mTags)[a];
@@ -709,7 +734,7 @@ void LWOImporter::LoadLWOPoints(unsigned int length)
 
 		// initialize all point referrers with the default values
 		mCurLayer->mPointReferrers.reserve	( regularSize + (regularSize>>2u) );
-		mCurLayer->mPointReferrers.resize	( regularSize, 0xffffffff );
+		mCurLayer->mPointReferrers.resize	( regularSize, UINT_MAX );
 	}
 	else mCurLayer->mTempPoints.resize( regularSize );
 
@@ -895,7 +920,7 @@ inline void LWOImporter::DoRecursiveVMAPAssignment(VMapEntry* base, unsigned int
 		base->rawData[idx*base->dims+i]= data[i];
 	}
 
-	if (0xffffffff != (i = refList[idx])) {
+	if (UINT_MAX != (i = refList[idx])) {
 		DoRecursiveVMAPAssignment(base,numRead,i,data);
 	}
 }
@@ -903,7 +928,7 @@ inline void LWOImporter::DoRecursiveVMAPAssignment(VMapEntry* base, unsigned int
 // ------------------------------------------------------------------------------------------------
 inline void AddToSingleLinkedList(ReferrerList& refList, unsigned int srcIdx, unsigned int destIdx)
 {
-	if(0xffffffff == refList[srcIdx])	{
+	if(UINT_MAX == refList[srcIdx])	{
 		refList[srcIdx] = destIdx;
 		return;
 	}
@@ -1028,12 +1053,13 @@ void LWOImporter::LoadLWO2VertexMap(unsigned int length, bool perPoly)
 						if (tmp == srcIdx)
 							break;
 					}
-					while ((tmp = refList[tmp]) != 0xffffffff);
-					if (tmp == 0xffffffff)
+					while ((tmp = refList[tmp]) != UINT_MAX);
+					if (tmp == UINT_MAX) {
 						continue;
+					}
 
 					had = true;
-					refList.resize(refList.size()+1, 0xffffffff);
+					refList.resize(refList.size()+1, UINT_MAX);
 						
 					idx = (unsigned int)pointList.size();
 					src.mIndices[i] = (unsigned int)pointList.size();
@@ -1095,7 +1121,8 @@ void LWOImporter::LoadLWO2Clip(unsigned int length)
 			int16_t offset = GetU2();  mFileBuffer+=4;
 			int16_t start  = GetU2();  mFileBuffer+=4;
 
-			std::string s;std::stringstream ss;
+			std::string s;
+			std::ostringstream ss;
 			GetS0(s,head->length);
 
 			head->length -= (unsigned int)s.length()+1;
@@ -1277,17 +1304,15 @@ void LWOImporter::LoadLWO2File()
 
 				AI_LWO_VALIDATE_CHUNK_LENGTH(head->length,LAYR,16);
 
+				// layer index.
+				layer.mIndex = GetU2();
+
 				// Continue loading this layer or ignore it? Check the layer index property
-				// NOTE: The first layer is the default layer, so the layer index is one-based now
-				if (0xffffffff != configLayerIndex && configLayerIndex != mLayers->size()-1)	{
+				if (UINT_MAX != configLayerIndex && (configLayerIndex-1) != layer.mIndex)	{
 					skip = true;
 				}
 				else skip = false;
 
-				// layer index. that's just for internal parenting, from the scope of a LWS file
-				// all layers are numbered in the oder in which they appear in the file
-				layer.mIndex = GetU2();
-
 				// pivot point
 				mFileBuffer += 2; /* unknown */
 				mCurLayer->mPivot.x = GetF4();
@@ -1311,6 +1336,10 @@ void LWOImporter::LoadLWO2File()
 				// optional: parent of this layer
 				if (mFileBuffer + 2 <= next)
 					layer.mParent = GetU2();
+				else layer.mParent = -1;
+
+				// Set layer skip parameter
+				layer.skip = skip;
 
 				break;
 			}
