@@ -1,9 +1,9 @@
 /*
 ---------------------------------------------------------------------------
-Open Asset Import Library (ASSIMP)
+Open Asset Import Library (assimp)
 ---------------------------------------------------------------------------
 
-Copyright (c) 2006-2010, ASSIMP Development Team
+Copyright (c) 2006-2012, assimp team
 
 All rights reserved.
 
@@ -20,10 +20,10 @@ conditions are met:
   following disclaimer in the documentation and/or other
   materials provided with the distribution.
 
-* Neither the name of the ASSIMP team, nor the names of its
+* Neither the name of the assimp team, nor the names of its
   contributors may be used to endorse or promote products
   derived from this software without specific prior
-  written permission of the ASSIMP Development Team.
+  written permission of the assimp team.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
@@ -47,6 +47,20 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "ObjFileParser.h"
 #include "ObjFileData.h"
 
+static const aiImporterDesc desc = {
+	"Wavefront Object Importer",
+	"",
+	"",
+	"surfaces not supported",
+	aiImporterFlags_SupportTextFlavour,
+	0,
+	0,
+	0,
+	0,
+	"obj"
+};
+
+
 namespace Assimp	{
 
 using namespace std;
@@ -84,11 +98,17 @@ bool ObjFileImporter::CanRead( const std::string& pFile, IOSystem*  pIOHandler ,
 	}
 	else //Check file Header
 	{
-		const char* tokens[] = {"mtllib","usemtl","vt ","vn ","o "};
-		return BaseImporter::SearchFileHeaderForToken(pIOHandler, pFile, tokens, 5);
+		static const char *pTokens[] = { "mtllib", "usemtl", "v ", "vt ", "vn ", "o ", "g ", "s ", "f " };
+		return BaseImporter::SearchFileHeaderForToken(pIOHandler, pFile, pTokens, 9 );
 	}
 }
 
+// ------------------------------------------------------------------------------------------------
+const aiImporterDesc* ObjFileImporter::GetInfo () const
+{
+	return &desc;
+}
+
 // ------------------------------------------------------------------------------------------------
 //	Obj-file import implementation
 void ObjFileImporter::InternReadFile( const std::string& pFile, aiScene* pScene, IOSystem* pIOHandler)
@@ -96,12 +116,12 @@ void ObjFileImporter::InternReadFile( const std::string& pFile, aiScene* pScene,
     DefaultIOSystem io;
     
 	// Read file into memory
-	const std::string mode  = "rb";
+	const std::string mode = "rb";
 	boost::scoped_ptr<IOStream> file( pIOHandler->Open( pFile, mode));
 	if (NULL == file.get())
 		throw DeadlyImportError( "Failed to open file " + pFile + ".");
 
-	// Get the filesize and vaslidate it, throwing an exception when failes
+	// Get the file-size and validate it, throwing an exception when fails
 	size_t fileSize = file->FileSize();
 	if( fileSize < 16)
 		throw DeadlyImportError( "OBJ-file is too small.");
@@ -175,7 +195,7 @@ void ObjFileImporter::CreateDataFromImport(const ObjFile::Model* pModel, aiScene
 // ------------------------------------------------------------------------------------------------
 //	Creates all nodes of the model
 aiNode *ObjFileImporter::createNodes(const ObjFile::Model* pModel, const ObjFile::Object* pObject, 
-									 unsigned int uiMeshIndex,
+									 unsigned int /*uiMeshIndex*/,
 									 aiNode *pParent, aiScene* pScene, 
 									 std::vector<aiMesh*> &MeshArray )
 {
@@ -183,14 +203,16 @@ aiNode *ObjFileImporter::createNodes(const ObjFile::Model* pModel, const ObjFile
 	if ( NULL == pObject )
 		return NULL;
 	
-	// Store older mesh size to be able to computate mesh offsets for new mesh instances
+	// Store older mesh size to be able to computes mesh offsets for new mesh instances
 	const size_t oldMeshSize = MeshArray.size();
 	aiNode *pNode = new aiNode;
+
+	pNode->mName = pObject->m_strObjName;
 	
+	// If we have a parent node, store it
 	if (pParent != NULL)
 		appendChildToParentNode(pParent, pNode);
 
-	
 	for ( unsigned int i=0; i< pObject->m_Meshes.size(); i++ )
 	{
 		unsigned int meshId = pObject->m_Meshes[ i ];
@@ -249,7 +271,23 @@ void ObjFileImporter::createTopology(const ObjFile::Model* pModel,
 	// Create faces
 	ObjFile::Mesh *pObjMesh = pModel->m_Meshes[ uiMeshIndex ];
 	ai_assert( NULL != pObjMesh );
-	pMesh->mNumFaces = static_cast<unsigned int>( pObjMesh->m_Faces.size() );
+
+	pMesh->mNumFaces = 0;
+	for (size_t index = 0; index < pObjMesh->m_Faces.size(); index++)
+	{
+		ObjFile::Face* const inp = pObjMesh->m_Faces[ index ];
+		if (inp->m_PrimitiveType == aiPrimitiveType_LINE) {
+			pMesh->mNumFaces += inp->m_pVertices->size() - 1;
+		}
+		else if (inp->m_PrimitiveType == aiPrimitiveType_POINT) {
+			pMesh->mNumFaces += inp->m_pVertices->size();
+		}
+		else {
+			++pMesh->mNumFaces;
+		}
+	}
+
+	unsigned int uiIdxCount = 0u;
 	if ( pMesh->mNumFaces > 0 )
 	{
 		pMesh->mFaces = new aiFace[ pMesh->mNumFaces ];
@@ -258,31 +296,40 @@ void ObjFileImporter::createTopology(const ObjFile::Model* pModel,
 			pMesh->mMaterialIndex = pObjMesh->m_uiMaterialIndex;
 		}
 
+		unsigned int outIndex = 0;
+
 		// Copy all data from all stored meshes
 		for (size_t index = 0; index < pObjMesh->m_Faces.size(); index++)
 		{
-			aiFace *pFace = &pMesh->mFaces[ index ];
-			const unsigned int uiNumIndices = (unsigned int) pObjMesh->m_Faces[ index ]->m_pVertices->size();
-			pFace->mNumIndices = (unsigned int) uiNumIndices;
-			if (pFace->mNumIndices > 0)
-			{
-				pFace->mIndices = new unsigned int[ uiNumIndices ];
-				ObjFile::Face::IndexArray *pIndexArray = pObjMesh->m_Faces[ index ]->m_pVertices;
-				ai_assert ( NULL != pIndexArray );
-				for ( size_t a=0; a<pFace->mNumIndices; a++ )
-				{
-					pFace->mIndices[ a ] = pIndexArray->at( a );
+			ObjFile::Face* const inp = pObjMesh->m_Faces[ index ];
+			if (inp->m_PrimitiveType == aiPrimitiveType_LINE) {
+				for(size_t i = 0; i < inp->m_pVertices->size() - 1; ++i) {
+					aiFace& f = pMesh->mFaces[ outIndex++ ];
+					uiIdxCount += f.mNumIndices = 2;
+					f.mIndices = new unsigned int[2];
 				}
+				continue;
 			}
-			else
-			{
-				pFace->mIndices = NULL;
+			else if (inp->m_PrimitiveType == aiPrimitiveType_POINT) {
+				for(size_t i = 0; i < inp->m_pVertices->size(); ++i) {
+					aiFace& f = pMesh->mFaces[ outIndex++ ];
+					uiIdxCount += f.mNumIndices = 1;
+					f.mIndices = new unsigned int[1];
+				}
+				continue;
+			}
+
+			aiFace *pFace = &pMesh->mFaces[ outIndex++ ];
+			const unsigned int uiNumIndices = (unsigned int) pObjMesh->m_Faces[ index ]->m_pVertices->size();
+			uiIdxCount += pFace->mNumIndices = (unsigned int) uiNumIndices;
+			if (pFace->mNumIndices > 0) {
+				pFace->mIndices = new unsigned int[ uiNumIndices ];			
 			}
 		}
 	}
 
 	// Create mesh vertices
-	createVertexArray(pModel, pData, uiMeshIndex, pMesh);
+	createVertexArray(pModel, pData, uiMeshIndex, pMesh, uiIdxCount);
 }
 
 // ------------------------------------------------------------------------------------------------
@@ -290,7 +337,8 @@ void ObjFileImporter::createTopology(const ObjFile::Model* pModel,
 void ObjFileImporter::createVertexArray(const ObjFile::Model* pModel, 
 										const ObjFile::Object* pCurrentObject, 
 										unsigned int uiMeshIndex,
-										aiMesh* pMesh)
+										aiMesh* pMesh,
+										unsigned int uiIdxCount)
 {
 	// Checking preconditions
 	ai_assert( NULL != pCurrentObject );
@@ -305,7 +353,7 @@ void ObjFileImporter::createVertexArray(const ObjFile::Model* pModel,
 		return;
 
 	// Copy vertices of this mesh instance
-	pMesh->mNumVertices = (unsigned int) pObjMesh->m_uiNumIndices;
+	pMesh->mNumVertices = uiIdxCount;
 	pMesh->mVertices = new aiVector3D[ pMesh->mNumVertices ];
 	
 	// Allocate buffer for normal vectors
@@ -316,31 +364,32 @@ void ObjFileImporter::createVertexArray(const ObjFile::Model* pModel,
 	if ( !pModel->m_TextureCoord.empty() && pObjMesh->m_uiUVCoordinates[0] )
 	{
 		pMesh->mNumUVComponents[ 0 ] = 2;
-		pMesh->mTextureCoords[ 0 ]   = new aiVector3D[ pMesh->mNumVertices ];
+		pMesh->mTextureCoords[ 0 ] = new aiVector3D[ pMesh->mNumVertices ];
 	}
 	
 	// Copy vertices, normals and textures into aiMesh instance
-	unsigned int newIndex = 0;
+	unsigned int newIndex = 0, outIndex = 0;
 	for ( size_t index=0; index < pObjMesh->m_Faces.size(); index++ )
 	{
-		// Get destination face
-		aiFace *pDestFace = &pMesh->mFaces[ index ];
-		
 		// Get source face
 		ObjFile::Face *pSourceFace = pObjMesh->m_Faces[ index ]; 
 
 		// Copy all index arrays
-		for ( size_t vertexIndex = 0; vertexIndex < pSourceFace->m_pVertices->size(); vertexIndex++ )
+		for ( size_t vertexIndex = 0, outVertexIndex = 0; vertexIndex < pSourceFace->m_pVertices->size(); vertexIndex++ )
 		{
 			const unsigned int vertex = pSourceFace->m_pVertices->at( vertexIndex );
-			ai_assert( vertex < pModel->m_Vertices.size() );
+			if ( vertex >= pModel->m_Vertices.size() ) 
+				throw DeadlyImportError( "OBJ: vertex index out of range" );
+			
 			pMesh->mVertices[ newIndex ] = pModel->m_Vertices[ vertex ];
 			
 			// Copy all normals 
-			if ( !pSourceFace->m_pNormals->empty() )
+			if ( !pSourceFace->m_pNormals->empty() && !pModel->m_Normals.empty())
 			{
 				const unsigned int normal = pSourceFace->m_pNormals->at( vertexIndex );
-				ai_assert( normal < pModel->m_Normals.size() );
+				if ( normal >= pModel->m_Normals.size() )
+					throw DeadlyImportError("OBJ: vertex normal index out of range");
+
 				pMesh->mNormals[ newIndex ] = pModel->m_Normals[ normal ];
 			}
 			
@@ -351,19 +400,61 @@ void ObjFileImporter::createVertexArray(const ObjFile::Model* pModel,
 				{
 					const unsigned int tex = pSourceFace->m_pTexturCoords->at( vertexIndex );
 					ai_assert( tex < pModel->m_TextureCoord.size() );
-					for ( size_t i=0; i < AI_MAX_NUMBER_OF_TEXTURECOORDS; i++ )
+					for ( size_t i=0; i < pMesh->GetNumUVChannels(); i++ )
 					{
-						if ( pMesh->mNumUVComponents[ i ] > 0 )
-						{
-							aiVector2D coord2d = pModel->m_TextureCoord[ tex ];
-							pMesh->mTextureCoords[ i ][ newIndex ] = aiVector3D( coord2d.x, coord2d.y, 0.0 );
-						}
+						if ( tex >= pModel->m_TextureCoord.size() )
+							throw DeadlyImportError("OBJ: texture coord index out of range");
+
+						aiVector2D coord2d = pModel->m_TextureCoord[ tex ];
+						pMesh->mTextureCoords[ i ][ newIndex ] = aiVector3D( coord2d.x, coord2d.y, 0.0 );
 					}
 				}
 			}
 
 			ai_assert( pMesh->mNumVertices > newIndex );
-			pDestFace->mIndices[ vertexIndex ] = newIndex;
+
+			// Get destination face
+			aiFace *pDestFace = &pMesh->mFaces[ outIndex ];
+
+			const bool last = ( vertexIndex == pSourceFace->m_pVertices->size() - 1 ); 
+			if (pSourceFace->m_PrimitiveType != aiPrimitiveType_LINE || !last) 
+			{
+				pDestFace->mIndices[ outVertexIndex ] = newIndex;
+				outVertexIndex++;
+			}
+
+			if (pSourceFace->m_PrimitiveType == aiPrimitiveType_POINT) 
+			{
+				outIndex++;
+				outVertexIndex = 0;
+			}
+			else if (pSourceFace->m_PrimitiveType == aiPrimitiveType_LINE) 
+			{
+				outVertexIndex = 0;
+
+				if(!last) 
+					outIndex++;
+
+				if (vertexIndex) {
+					if(!last) {
+						pMesh->mVertices[ newIndex+1 ] = pMesh->mVertices[ newIndex ];
+						if ( !pSourceFace->m_pNormals->empty() && !pModel->m_Normals.empty()) {
+							pMesh->mNormals[ newIndex+1 ] = pMesh->mNormals[newIndex ];
+						}
+						if ( !pModel->m_TextureCoord.empty() ) {
+							for ( size_t i=0; i < pMesh->GetNumUVChannels(); i++ ) {
+								pMesh->mTextureCoords[ i ][ newIndex+1 ] = pMesh->mTextureCoords[ i ][ newIndex ];
+							}
+						}
+						++newIndex;
+					}
+
+					pDestFace[-1].mIndices[1] = newIndex;
+				}
+			}
+			else if (last) {
+				outIndex++;
+			}
 			++newIndex;
 		}
 	}	
@@ -399,14 +490,14 @@ void ObjFileImporter::createMaterials(const ObjFile::Model* pModel, aiScene* pSc
 
 	const unsigned int numMaterials = (unsigned int) pModel->m_MaterialLib.size();
 	pScene->mNumMaterials = 0;
-	if ( pModel->m_MaterialLib.empty() )
+	if ( pModel->m_MaterialLib.empty() ) {
+		DefaultLogger::get()->debug("OBJ: no materials specified");
 		return;
+	}
 	
 	pScene->mMaterials = new aiMaterial*[ numMaterials ];
 	for ( unsigned int matIndex = 0; matIndex < numMaterials; matIndex++ )
-	{
-		Assimp::MaterialHelper* mat = new Assimp::MaterialHelper;
-		
+	{		
 		// Store material name
 		std::map<std::string, ObjFile::Material*>::const_iterator it;
 		it = pModel->m_MaterialMap.find( pModel->m_MaterialLib[ matIndex ] );
@@ -415,6 +506,7 @@ void ObjFileImporter::createMaterials(const ObjFile::Model* pModel, aiScene* pSc
 		if ( pModel->m_MaterialMap.end() == it )
 			continue;
 
+		aiMaterial* mat = new aiMaterial;
 		ObjFile::Material *pCurrentMaterial = (*it).second;
 		mat->AddProperty( &pCurrentMaterial->MaterialName, AI_MATKEY_NAME );
 
@@ -433,8 +525,9 @@ void ObjFileImporter::createMaterials(const ObjFile::Model* pModel, aiScene* pSc
 			break;
 		default:
 			sm = aiShadingMode_Gouraud;
-			DefaultLogger::get()->error("OBJ/MTL: Unexpected illumination model (0-2 recognized)");
+			DefaultLogger::get()->error("OBJ: unexpected illumination model (0-2 recognized)");
 		}
+	
 		mat->AddProperty<int>( &sm, 1, AI_MATKEY_SHADING_MODEL);
 
 		// multiplying the specular exponent with 2 seems to yield better results
@@ -489,6 +582,7 @@ void ObjFileImporter::appendChildToParentNode(aiNode *pParent, aiNode *pChild)
 	// Assign parent to child
 	pChild->mParent = pParent;
 	size_t sNumChildren = 0;
+	(void)sNumChildren; // remove warning on release build
 	
 	// If already children was assigned to the parent node, store them in a 
 	std::vector<aiNode*> temp;
